----------------------------------------------------------------------------------
-- Company: 
-- Engineer: 
-- 
-- Create Date: 03/19/2025 11:55:14 AM
-- Design Name: 
-- Module Name: ecgUnit - Behavioral
-- Project Name: 
-- Target Devices: 
-- Tool Versions: 
-- Description: 
-- 
-- Dependencies: 
-- 
-- Revision:
-- Revision 0.01 - File Created
-- Additional Comments:
-- 
----------------------------------------------------------------------------------


library IEEE;
use IEEE.STD_LOGIC_1164.ALL;

-- Uncomment the following library declaration if using
-- arithmetic functions with Signed or Unsigned values
--use IEEE.NUMERIC_STD.ALL;

-- Uncomment the following library declaration if instantiating
-- any Xilinx leaf cells in this code.
--library UNISIM;
--use UNISIM.VComponents.all;

entity ecgUnit is
port(
    I_clock               : in  std_logic;  -- global clock
    I_reset               : in  std_logic;  -- asynchronous global reset
    I_inputSample         : in  std_logic_vector(10 downto 0);  -- 8 bit input sample
    I_inputSampleValid    : in  std_logic;
    O_filteredSample      : out std_logic_vector(10 downto 0)  -- filtered sample
    );
end ecgUnit;

architecture Behavioral of ecgUnit is

component fsm is
    port (
        I_Clock 		        : in  std_logic;
        I_reset 	           	: in  std_logic;
        I_inputSampleValid      : in  std_logic;
        I_processing1Done       : in  std_logic;
        I_processing2Done       : in  std_logic;
        I_processing2PrimeDone  : in  std_logic;
        I_processing3Done       : in  std_logic;
        O_loadShift1	        : out std_logic;
        O_loadShift2	        : out std_logic;
        O_loadShift3	        : out std_logic;
        O_initAddress	      	: out std_logic;
        O_incrAddress          	: out std_logic;
        O_initSum	            : out std_logic;
        O_loadSum		        : out std_logic;
        O_loadOutput	        : out std_logic;
        O_filter	            : out std_logic_vector(1 downto 0);
        O_FilteredSampleDone    : in std_logic);
  end component fsm;

  component OperativeUnit is
    port (
        I_clock          : in  std_logic;   -- global clock
        I_reset          : in  std_logic;   -- asynchronous global reset
        I_inputSample    : in  std_logic_vector(9 downto 0);  -- 8 bit input sample
        I_loadShift1     : in  std_logic;  -- Control signal to load the input sample in the sample shift register and shift the register
        I_loadShift2     : in  std_logic;  -- Control signal to load the input sample in the sample shift register and shift the register
        I_loadShift3     : in  std_logic;  -- Control signal to load the input sample in the sample shift register and shift the register
        I_initAddress    : in  std_logic;  -- Control signal to initialize register read address
        I_incrAddress    : in  std_logic;  -- Control signal to increment register read address
        I_initSum        : in  std_logic;  -- Control signal to initialize the MAC register
        I_loadSum        : in  std_logic;  -- Control signal to load the MAC register;
        I_loadY          : in  std_logic;   -- Control signal to load Y register
        I_Filter         : in std_logic_vector(1 downto 0);
        I_FilteredSampleDone : in std_logic;
        O_processingDone1 : out std_logic;   -- Indicate that processing is done
        O_processingDone2 : out std_logic;   -- Indicate that processing is done
        O_processingDone2b : out std_logic;   -- Indicate that processing is done
        O_processingDone3 : out std_logic;   -- Indicate that processing is done
        O_Y              : out std_logic_vector(9 downto 0)   -- filtered sample
    );
  end component OperativeUnit;
  
  
  signal SC_processingDone1 : std_logic;
  signal SC_processingDone2 : std_logic;
  signal SC_processingDone2b : std_logic;
  signal SC_processingDone3 : std_logic;
  signal SC_loadShift1      : std_logic;
  signal SC_loadShift2      : std_logic;
  signal SC_loadShift3      : std_logic;
  signal SC_initAddress    : std_logic;
  signal SC_incrAddress    : std_logic;
  signal SC_initSum        : std_logic;
  signal SC_loadSum        : std_logic;
  signal SC_loadY          : std_logic;
  signal SC_Filter          : std_logic_vector(1 downto 0);
  signal SC_FilteredSampleDone : std_logic;

begin

controlUnit_1 : entity work.fsm
    port map (
        I_Clock 		          => I_clock,      
        I_reset 	              => I_reset,  
        I_inputSampleValid        => I_inputSampleValid,
        I_processing1Done         => SC_processingDone1,
        I_processing2Done         => SC_processingDone2,
        I_processing2PrimeDone    => SC_processingDone2b,
        I_processing3Done         => SC_processingDone3,
        O_loadShift1	          => SC_loadShift1,
        O_loadShift2	          => SC_loadShift2,
        O_loadShift3	          => SC_loadShift3,
        O_initAddress	      	  => SC_initAddress,
        O_incrAddress             => SC_incrAddress,
        O_initSum	              => SC_initSum,
        O_loadSum		          => SC_loadSum,
        O_loadOutput	          => SC_loadY,
        O_filter	              => SC_Filter,
        O_FilteredSampleDone      => SC_FilteredSampleDone
        );

OpUnit : entity work.OperativeUnit
    port map (
        I_clock                 => I_clock,   
        I_reset                 => I_reset,
        I_inputSample           => I_inputSample,
        I_loadShift1            => SC_loadShift1,
        I_loadShift2            => SC_loadShift2,
        I_loadShift3            => SC_loadShift3,
        I_initAddress           => SC_initAddress,
        I_incrAddress           => SC_incrAddress,
        I_initSum               => SC_initSum,
        I_loadSum               => SC_loadSum,
        I_loadY                 => SC_loadY,  
        I_Filter                => SC_Filter,
        I_FilteredSampleDone    => SC_FilteredSampleDone,
        O_processingDone1       => SC_ProcessingDone1,
        O_processingDone2       => SC_ProcessingDone2,
        O_processingDone2b      => SC_ProcessingDone2b,
        O_processingDone3       => SC_ProcessingDone3,
        O_Y                     => O_filteredSample
        );
        


end Behavioral;
