----------------------------------------------------------------------------------
-- Company: 
-- Engineer: 
-- 
-- Create Date: 03/19/2025 09:52:24 AM
-- Design Name: 
-- Module Name: fsm - Behavioral
-- Project Name: 
-- Target Devices: 
-- Tool Versions: 
-- Description: 
-- 
-- Dependencies: 
-- 
-- Revision:
-- Revision 0.01 - File Created
-- Additional Comments:
-- 
----------------------------------------------------------------------------------


library IEEE;
use IEEE.STD_LOGIC_1164.ALL;
use IEEE.numeric_std.all;

entity fsm is
    port (
        I_Clock 		: in  std_logic;
        I_reset 		: in  std_logic;
        I_inputSampleValid      : in  std_logic;
        I_processing1Done       : in  std_logic;
        I_processing2Done       : in  std_logic;
        I_processing2PrimeDone  : in  std_logic;
        I_processing3Done       : in  std_logic;
        O_loadShift1	        : out std_logic;
        O_loadShift2	        : out std_logic;
        O_loadShift3	        : out std_logic;
        O_initAddress		: out std_logic;
        O_incrAddress     	: out std_logic;
        O_initSum	        : out std_logic;
        O_loadSum		: out std_logic;
        O_loadOutput	        : out std_logic;
        O_filter	        : out std_logic_vector(1 downto 0);
        O_FilteredSampleDone : out std_logic
        );
end entity fsm;

architecture a_fsm of fsm is

    type T_State is (WAIT_SAMPLE, STORE, PROCESSING_LOOP_1, OUTPUT_1, STORE1, PROCESSING_LOOP_2, OUTPUT_2, STORE2, PROCESSING_LOOP_2b, OUTPUT_2b, STORE2b,PROCESSING_LOOP_3, OUTPUT, WAIT_END_SAMPLE);
    
    signal SR_present : T_State;
    signal SC_futur   : T_State;

begin

    process(I_Clock, I_reset) is
    begin
        if I_reset = '1' then
            SR_present <= WAIT_SAMPLE;
        elsif(rising_edge(I_Clock)) then
            SR_present <= SC_futur;
        end if;
    end process;


    process(SR_present, I_reset, I_inputSampleValid, I_processing1Done, I_processing2Done, I_processing2PrimeDone, I_processing3Done)
    begin
        case SR_present is

            when WAIT_SAMPLE =>
                if(I_inputSampleValid = '1') then
                    SC_futur <= STORE;
                else
                    SC_futur <= WAIT_SAMPLE;
                end if;

            when STORE =>
                SC_futur <= PROCESSING_LOOP_1;

            when PROCESSING_LOOP_1 =>
                if(I_processing1Done = '1') then
                    SC_futur <= OUTPUT_1;
                else
                    SC_futur <= PROCESSING_LOOP_1;
                end if;

            when OUTPUT_1 =>
                SC_futur <= STORE1;
            
            when STORE1 =>
                SC_futur <= PROCESSING_LOOP_2;

	    when PROCESSING_LOOP_2 =>
                if(I_processing2Done = '1') then
                    SC_futur <= OUTPUT_2;
                else
                    SC_futur <= PROCESSING_LOOP_2;
                end if;

            when OUTPUT_2 =>
                SC_futur <= STORE2;
                
            when STORE2 =>
                SC_futur <= PROCESSING_LOOP_2b;

	    when PROCESSING_LOOP_2b =>
                if(I_processing2PrimeDone = '1') then
                    SC_futur <= OUTPUT_2b;
                else
                    SC_futur <= PROCESSING_LOOP_2b;
                end if;

            when OUTPUT_2b =>
                SC_futur <= STORE2b;
            
            when STORE2b =>
                SC_futur <= PROCESSING_LOOP_3;

	    when PROCESSING_LOOP_3 =>
                if(I_processing3Done = '1') then
                    SC_futur <= OUTPUT;
                else
                    SC_futur <= PROCESSING_LOOP_3;
                end if;

            when OUTPUT =>
                SC_futur <= WAIT_END_SAMPLE;

            when WAIT_END_SAMPLE =>
                if(I_inputSampleValid = '0') then  
                    SC_futur <= WAIT_SAMPLE;
                else
                    SC_futur <= WAIT_END_SAMPLE;
                end if;

        end case;
    end process;


    process(SR_present, I_reset, I_inputSampleValid, I_processing1Done, I_processing2Done, I_processing2PrimeDone, I_processing3Done)
    begin

        -- default output values

	    O_loadShift1   <= '0';
	    O_loadShift2   <= '0';
	    O_loadShift3   <= '0';
        O_initAddress <= '0';
        O_incrAddress <= '0';
        O_initSum     <= '0';
        O_loadSum     <= '0';
        O_loadOutput   <= '0';
        O_filter      <= "00";
        O_FilteredSampleDone <= '0';

        case SR_present is

            when STORE =>
                O_loadShift1   <= '1';
                O_initAddress <= '1';
                O_initSum     <= '1';
                O_filter      <= "00";

            when PROCESSING_LOOP_1 =>
                O_incrAddress <= '1';
                O_loadSum     <= '1';
                O_filter      <= "00";

            when OUTPUT_1 =>
                O_loadOutput  <= '1';
                
            when STORE1 =>
                O_loadShift2   <= '1';
                O_initAddress <= '1';
                O_initSum     <= '1';
                O_filter      <= "01";

            when PROCESSING_LOOP_2 =>
                O_incrAddress <= '1';
                O_loadSum     <= '1';
                O_filter      <= "01";

            when OUTPUT_2 =>
                O_loadOutput  <= '1';               
                
            when STORE2 =>
                O_loadShift3   <= '1';
                O_initAddress <= '1';
                O_initSum     <= '1';
                O_filter      <= "10";

            when PROCESSING_LOOP_2b =>
                O_incrAddress <= '1';
                O_loadSum     <= '1';
                O_filter      <= "10";

            when OUTPUT_2b =>
                O_loadOutput  <= '1';
                
            when STORE2b =>
                O_initAddress <= '1';                                       
                O_initSum     <= '1';
                O_filter      <= "11";
        
            when PROCESSING_LOOP_3 =>
                O_incrAddress <= '1';
                O_loadSum     <= '1';
                O_filter      <= "11";

            when OUTPUT =>
                O_FilteredSampleDone <= '1';    
                O_filter <= "11";      
                
            when others =>
                O_loadShift1   <= '0';
	            O_loadShift2   <= '0';
	            O_loadShift3   <= '0';
                O_initAddress <= '0';
                O_incrAddress <= '0';
                O_initSum     <= '0';
                O_loadSum     <= '0';
                O_loadOutput   <= '0';
                O_filter      <= "00";
                O_FilteredSampleDone <= '0';             
            
        end case;
    end process;
end a_fsm;